@@ -255,7 +255,7 @@ int get_unary_uuid(struct bt_list_head *head, unsigned char *uuid)
 				|| i != 0)
 			return -EINVAL;
 		src_string = node->u.unary_expression.u.string;
-		ret = babeltrace_uuid_parse(src_string, uuid);
+		ret = bt_uuid_parse(src_string, uuid);
 	}
 	return ret;
 }
@@ -2178,7 +2178,7 @@ int ctf_trace_declaration_visit(FILE *fd, int depth, struct ctf_node *node, stru
 				goto error;
 			}
 			if (CTF_TRACE_FIELD_IS_SET(trace, uuid)
-				&& babeltrace_uuid_compare(uuid, trace->uuid)) {
+				&& bt_uuid_compare(uuid, trace->uuid)) {
 				fprintf(fd, "[error] %s: uuid mismatch\n", __func__);
 				ret = -EPERM;
 				goto error;
@@ -2792,6 +2792,22 @@ int ctf_env_declaration_visit(FILE *fd, int depth, struct ctf_node *node,
 			env->domain[TRACER_ENV_LEN - 1] = '\0';
 			printf_verbose("env.domain = \"%s\"\n", env->domain);
 			g_free(right);
+		} else if (!strcmp(left, "tracer_name")) {
+			char *right;
+
+			if (env->tracer_name[0]) {
+				fprintf(fd, "[warning] %s: duplicated env tracer_name\n", __func__);
+				goto error;	/* ret is 0, so not an actual error, just warn. */
+			}
+			right = concatenate_unary_strings(&node->u.ctf_expression.right);
+			if (!right) {
+				fprintf(fd, "[warning] %s: unexpected unary expression for env tracer_name\n", __func__);
+				goto error;	/* ret is 0, so not an actual error, just warn. */
+			}
+			strncpy(env->tracer_name, right, TRACER_ENV_LEN);
+			env->tracer_name[TRACER_ENV_LEN - 1] = '\0';
+			printf_verbose("env.tracer_name = \"%s\"\n", env->tracer_name);
+			g_free(right);
 		} else if (!strcmp(left, "sysname")) {
 			char *right;
 
